XIncludeFile "vector.pb"
XIncludeFile "Math.pb"

Structure count
   particle.l
   constraint.l
   entity.l
   collision.l
EndStructure

Structure particle
   p.vect
   oldp.vect
   v.vect
   a.vect
   m.f
   id.l
   collision.w
   *parent.entity
   type.l
   r.l
EndStructure

Global NewList particle.particle() 

Structure constraint
   *p1.particle
   *p2.particle
   e.f ;elasticity
   length.f
   id.l
   collision.w
   *parent.entity
EndStructure

Global NewList constraint.constraint()

Structure entity
   vertexcount.l
   edgecount.l
   *vertex.particle[1000]
   *edge.constraint[1000]
   *data  ;user defined data
   center.vect
   min.vect ;min -
   max.vect ;max for the bounding box
   m.f ;mass
   f.f ;friction
   collision.w
   id.l
   type.s
EndStructure

Global NewList entity.entity()

Structure halfspace
   p.vect
   v.vect
   n0.vect
   type.s
EndStructure

Global NewList halfspace.halfspace()

Structure collision ; every collision is is stored for 1 frame to pass data to friction handling or other things
   type.w
   *v1.particle
   *v2.particle
   *p.particle
   colv.vect  ;collisionvekor for correction
   relativspeed.f ; the relativ speed with witch the 2 objects colide with each other
   colspeed.f ; the relativ speed in colv direction
   id.l
EndStructure

Global NewList collision.collision()

Structure CollisionRemark ;before detecting collision, these remarks are calculated and stand for possible collisions
   *e1.entity
   *e2.entity
EndStructure

Global NewList CollisionRemark.CollisionRemark()

Global count.count
Global timestep.f; delta t
timestep=0.5
Global iterations.l
iterations=10
Global gravity.vect
gravity\y=0.5
gravity\x=0
Global particleid.l
particleid=1
Global constraintid.l
constraintid=1
Global entityid.l
entityid=1
Global collisionid.l
collisionid=1
Global ProjectedMin.f
Global ProjectedMax.f
Global r.f ; this is the air-friction coefficient
r=0.005
Global intelligenttolleration.w
intelligenttolleration=2
Global maxspeed.f
maxspeed=50
Global *neutralentity.entity

Procedure InitPhysics()
   *neutralentity=AddElement(entity())
   entity()\m=1
   entity()\id=1
   entityid+1
EndProcedure

Procedure.l CountParticles()
    ProcedureReturn ListSize(particle())
EndProcedure
Procedure.l CountConstraint()
    ProcedureReturn ListSize(constraint())
EndProcedure
Procedure.l CountEntity()
   ProcedureReturn ListSize(entity())
EndProcedure
Procedure.l CountCollision()
   ProcedureReturn ListSize(collision())
EndProcedure

Procedure CountAll()
   count\particle=CountParticles()
   count\constraint=CountConstraint()
   count\entity=CountEntity()
   count\collision=CountCollision()
EndProcedure
Procedure CollisionIdentic(*c.collision)
   Protected i.l
   ForEach collision()
      If collision()\type=2 Or *c\type=2
         If collision()\p\id=*c\p\id
            ProcedureReturn 1
         EndIf
      ElseIf collision()\v1\id=*c\v1\id And collision()\p\id=*c\p\id
         ProcedureReturn 1
      ElseIf collision()\v1\id=*c\v2\id And collision()\p\id=*c\p\id
         ProcedureReturn 1 
      EndIf
   Next
   ProcedureReturn 0
EndProcedure

Procedure CreateParticle(*parent.entity,px.f,py.f)
   AddElement(particle())
   count\particle+1
   particle()\p\x=px
   particle()\p\y=py
   particle()\oldp\x=px
   particle()\oldp\y=py
   particle()\id=particleid
   particleid+1
   particle()\r=0
   particle()\parent=*parent
   If *parent\id<>1
      particle()\parent\vertexcount+1
      particle()\parent\vertex[particle()\parent\vertexcount]=particle()
   EndIf
   vect(particle()\a,gravity\x,gravity\y)
   ProcedureReturn particle()
EndProcedure
Procedure CreateConstraint(*parent.entity,*p1.particle,*p2.particle,e.f)
   If *p1<>*p2
      AddElement(constraint())
      constraint()\p1=*p1
      constraint()\p2=*p2
      constraint()\length=Sqr((*p2\p\x-*p1\p\x)*(*p2\p\x-*p1\p\x)+(*p2\p\y-*p1\p\y)*(*p2\p\y-*p1\p\y))  
      constraint()\e=e
      constraint()\id=constraintid
      constraintid+1
      constraint()\parent=*parent
      If *parent\id<>1
         *parent\edgecount+1
         *parent\edge[*parent\edgecount]=constraint()
      EndIf
      count\constraint+1
      ProcedureReturn constraint()
   EndIf
EndProcedure
Procedure CreatePEntity(m.f,f.f)
   AddElement(entity())
   count\entity+1
   entity()\m=m
   entity()\f=f
   entity()\id=entityid
   entityid+1
   ProcedureReturn entity()
EndProcedure
Procedure AddHalfSpace(px,py,vx.f,vy.f)
   AddElement(halfspace())
   halfspace()\p\x=px
   halfspace()\p\y=py
   halfspace()\v\x=vx
   halfspace()\v\y=vy
   vect(halfspace()\n0,vy,-vx)
   Protected length.f
   length=length(halfspace()\n0)
   vect(halfspace()\n0,halfspace()\n0\x/length,halfspace()\n0\y/length)
EndProcedure
Procedure AddCollision(*v1.particle,*v2.particle,*p.particle,*colv.vect)
   AddElement(collision())
   collision()\v1=*v1
   collision()\v2=*v2
   collision()\p=*p
   collision()\colv\x=*colv\x
   collision()\colv\y=*colv\y
   collision()\id=collisionid
   collision()\type=0
   collisionid+1
   count\collision+1
EndProcedure
Procedure AddCollisionRemark(*e1.entity,*e2.entity)
   AddElement(CollisionRemark())
   CollisionRemark()\e1=*e1
   CollisionRemark()\e2=*e2
EndProcedure

Procedure DeleteConstraint(*c.constraint)
   ForEach entity()
      For i=1 To entity()\edgecount
         If entity()\edge[i]=*c
            ;a lot of things to to
         EndIf
      Next
   Next
   ChangeCurrentElement(constraint(),*c)
   DeleteElement(constraint())
EndProcedure
Procedure DeleteParticle(*p.Particle)
   ForEach entity()
      For i=1 To entity()\vertexcount
         If entity()\vertex[i]=*c
            ;a lot of things to to
         EndIf
      Next
   Next
   ForEach constraint()
      If constraint()\p1=*p Or constraint()\p2=*p
         DeleteElement(constraint())
      EndIf
   Next
   ChangeCurrentElement(Particle(), *p)
   DeleteElement(Particle())
 EndProcedure
Procedure deleteentity(*e.entity)
   ForEach CollisionRemark()
      If CollisionRemark()\e1=*e Or CollisionRemark()\e2=*e
         DeleteElement(CollisionRemark())
      EndIf
   Next
   ForEach Collision()
      If collision()\p\parent=*e Or collision()\v1\parent=*e Or collision()\v2\parent=*e
         DeleteElement(Collision())
      EndIf
   Next
   For i=1 To *e\edgecount
      ChangeCurrentElement(constraint(),*e\edge[i])
      DeleteElement(constraint())
   Next
   For i=1 To *e\vertexcount
      ForEach constraint()
         If constraint()\p1=*e\vertex[i] Or constraint()\p2=*e\vertex[i]
            DeleteElement(constraint())
         EndIf
      Next
      ChangeCurrentElement(particle(),*e\vertex[i])
      DeleteElement(particle())
   Next 
   ChangeCurrentElement(entity(),*e)
   DeleteElement(entity())
EndProcedure

;these are the important Verlet Integration Procedures
Procedure UpdateVerlet()
   Protected old.vect
   Protected v.vect
   Protected length.f
   ForEach particle()
      If particle()\parent\m<>0
         vect(old,particle()\p\x,particle()\p\y)
         particle()\v\x=particle()\p\x-particle()\oldp\x+particle()\a\x*(timestep*timestep)
         particle()\v\y=particle()\p\y-particle()\oldp\y+particle()\a\y*(timestep*timestep)
         vect(particle()\oldp,old\x,old\y)
         
        ;air friction vector
         vect(particle()\v,particle()\v\x*(1-r),particle()\v\y*(1-r))   
         
         vect(v,particle()\v\x,particle()\v\y)
         length=length(v)
         If length>maxspeed
            vect(v,v\x/length*(length-maxspeed),v\y/length*(length-maxspeed))
            vect(particle()\v,v\x,v\y)        
         EndIf
     EndIf
   Next
EndProcedure
Procedure UpdateSpeed()
   ForEach Particle() : With Particle()
      If \parent\m<>0     
         \p\x+\v\x/iterations
         \p\y+\v\y/iterations
      EndIf
   EndWith : Next
EndProcedure
Procedure UpdateConstraints()
   Protected DeltaX.f
   Protected DeltaY.f
   Protected DeltaLength.f
   Protected Diff.f
   Protected x.f
      ForEach constraint()
         DeltaX=constraint()\p2\p\x-constraint()\p1\p\x
         DeltaY=constraint()\p2\p\y-constraint()\p1\p\y
         DeltaLength=Sqr(DeltaX*DeltaX+DeltaY*DeltaY)
         Diff=(DeltaLength-constraint()\length)/DeltaLength
         If constraint()\p1\parent\m<>0
            constraint()\p1\p\x=constraint()\p1\p\x+DeltaX*Diff/2*constraint()\e
            constraint()\p1\p\y=constraint()\p1\p\y+DeltaY*Diff/2*constraint()\e
         EndIf
         If constraint()\p2\parent\m<>0
            constraint()\p2\p\x=constraint()\p2\p\x-DeltaX*Diff/2*constraint()\e
            constraint()\p2\p\y=constraint()\p2\p\y-DeltaY*Diff/2*constraint()\e
         EndIf
      Next
EndProcedure

Procedure.f PointInHalfspace(p1x.f,p1y.f,p2x.f,p2y.f,PX.f,PY.f,ox=0,oy=0)
   Protected p.vect
   Protected n0.vect
   Protected length.f
   Protected distance.f
   Protected d.f
   Protected h.vect
   vect(p,p1x,p1y)
   vect(n0,-(p2y-p1y),p2x-p1x)
   length=length(n0)
   vect(n0,n0\x/length,n0\y/length)
   vect(h,PX-p\x,PY-p\y)
   distance=dotproduct(h,n0)
   If ox<>0 Or oy<>0
      vect(p,p1x,p1y)
      vect(n0,-(p2y-p1y),p2x-p1x)
      length=length(n0)
      vect(n0,n0\x/length,n0\y/length)
      vect(h,ox-p\x,oy-p\y)
      d=dotproduct(h,n0)
      If d<0
         distance*-1
      EndIf
   EndIf
   ProcedureReturn distance
EndProcedure
Procedure PointInEntity(*e.entity,px,py)
   Protected Dim *p.particle(*e\vertexcount)
   Protected Dim a.f(*e\vertexcount)
   Protected ha.f
   Protected *hp.particle
   For i=1 To *e\vertexcount
      a(i)=arc(*e\center\x,*e\center\y,*e\vertex[i]\p\x,*e\vertex[i]\p\y)
      *p(i)=*e\vertex[i]
   Next
   
   Protected sort.b
   Protected limit.w
   limit=0
   Repeat
      sort=1
      For i=1 To *e\vertexcount-1
         If a(i)>a(i+1)
            ha=a(i)
            a(i)=a(i+1)
            a(i+1)=ha
         
            *hp=*p(i)
            *p(i)=*p(i+1)
            *p(i+1)=*hp
            sort=0
         EndIf
      Next
      limit+1
   Until sort=1 Or limit=100
   
   Protected true.w
   true=0
   For i=1 To *e\vertexcount-1
      If PointInHalfspace(*p(i)\p\x,*p(i)\p\y,*p(i+1)\p\x,*p(i+1)\p\y,px,py)>0
         true+1
      EndIf
   Next
   If PointInHalfspace(*p(*e\vertexcount)\p\x,*p(*e\vertexcount)\p\y,*p(1)\p\x,*p(1)\p\y,px,py)>0
      true+1
   EndIf
   If true=*e\vertexcount Or true=0
      ProcedureReturn 1
   Else
      ProcedureReturn 0
   EndIf 
EndProcedure
Procedure NearestParticle(x,y)
   If ListSize(particle())<=0
      ProcedureReturn 0
   EndIf
   *nearest.particle
   distance.d
   mindistance.d
   *nearest=particle()
   mindistance=Sqr((x-particle()\p\x)*(x-particle()\p\x)+(y-particle()\p\y)*(y-particle()\p\y))
   ForEach particle()
      distance=Sqr((x-particle()\p\x)*(x-particle()\p\x)+(y-particle()\p\y)*(y-particle()\p\y))
      If distance<=mindistance
         mindistance=distance
         *nearest=particle()
      EndIf
   Next
   ProcedureReturn *nearest
EndProcedure

Procedure UpdateParticleHalfspaceCollision()
   Protected n0.vect
   Protected length.f
   Protected distance.f
   Protected h.vect
   ResetList(halfspace())
   While NextElement(halfspace())
      ForEach particle()
         If particle()\parent\m<>0
            vect(n0,halfspace()\n0\x,halfspace()\n0\y)
            vect(h,particle()\p\x-halfspace()\p\x,particle()\p\y-halfspace()\p\y)
            distance=dotproduct(h,n0)
            If distance<0
                If distance<-10
                   distance=-10
               EndIf
               vect(h,n0\x*distance,n0\y*distance)
               vect(particle()\p,particle()\p\x-h\x,particle()\p\y-h\y)
               particle()\collision=1
               Protected col.collision
               col\p=particle()
               col\type=2

               If collisionidentic(col)=0
                  Protected speedvect.vect
                  vect(speedvect,(col\p\p\x-col\p\oldp\x),(col\p\p\y-col\p\oldp\y))
                  Protected DotPro.f 
                  DotPro=DotProduct(n0,speedvect)
                  Protected speedvect2.vect
                  vect(speedvect2,speedvect\x*DotPro,speedvect\y*DotPro)
      
                  addcollision(0,0,particle(),h)
                  collision()\type=2
                  collision()\relativspeed=length(speedvect)
                  collision()\colspeed=length(speedvect2)
               EndIf
            EndIf
         EndIf
      Next
   Wend
EndProcedure
Procedure UpdateFriction()
   Protected *v2.particle
   Protected *v1.particle
   Protected *p.particle
   Protected DotP.f
   Protected friction.vect
   Protected v.vect;relative geschw.
   ForEach collision()
      If collision()\type=0
         *v1=collision()\v1
         *v2=collision()\v2
         *p=collision()\p
      
         ;Friction / Reibungs Richtungsvektor wird festgelegt & mit collidierenden Koefizienten verrechnet (Vektor wir noch nicht hinzugefgt)

         vect(friction,(*v2\p\x-*v1\p\x),(*v2\p\y-*v1\p\y))
         Normalize(friction) ;Richtung berrechnen
      
         Protected v1v.vect
         vect(v1v,(*v1\p\x-*v1\oldp\x)-(*p\p\x-*p\oldp\x),(*v1\p\y-*v1\oldp\y)-(*p\p\y-*p\oldp\y))
        vect(v1v,v1v\x*(Ratio),v1v\y*(Ratio))
        Protected v2v.vect
        vect(v2v,(*v2\p\x-*v2\oldp\x)-(*p\p\x-*p\oldp\x),(*v2\p\y-*v2\oldp\y)-(*p\p\y-*p\oldp\y))
         vect(v2v,v2v\x*(1-Ratio),v2v\y*(1-Ratio))
      
        ;Relative Object-Geschwindigkeit!!!
         vect(v,(v1v\x+v2v\x),(v1v\y+v2v\y));hierbei wird noch *v1 representativ fr den Reibungspunkt an der Seite v1-v2 genommen ?
        DotP=DotProduct(v,friction)
        vect(v,friction\x*DotP,friction\y*DotP)
      
        vect(friction,friction\x**p\parent\f**v1\parent\f,friction\y**p\parent\f**v1\parent\f)
         Protected l.f
         l=Length(collision()\colv)
         vect(friction,friction\x*l,friction\y*l)
         
         vect(friction,friction\x/iterations,friction\y/iterations)
      
         If (*p\parent\m=0 Or *v1\parent\m=0)
            If Abs(Length(friction))>Abs(Length(v))
              friction\x=-v\x
              friction\y=-v\y
            EndIf
         ElseIf Abs(Length(friction))>Abs(Length(v))/3
           friction\x=-v\x/3
           friction\y=-v\y/3
         EndIf
      
         If  Abs(friction\x)>Abs(friction\y)
            If (v\x>0 And friction\x>0) Or (v\x<0 And friction\x<0)
               vect(friction,friction\x*-1,friction\y*-1)
            EndIf
         Else
            If (v\y>0 And friction\y>0) Or (v\y<0 And friction\y<0)
               vect(friction,friction\x*-1,friction\y*-1)
            EndIf
         EndIf
         
            Protected mges.f
            mges=*v1\parent\m+*p\parent\m
            Protected vm.f
            vm=(*p\parent\m/mges)*2
            Protected pm.f
            pm=(*v1\parent\m/mges)*2
            
         If *v1\parent\m<>0 And *p\parent\m<>0 
            vect(*v1\p,*v1\p\x+Friction\x*vm,*v1\p\y+Friction\y*vm)
            vect(*v2\p,*v2\p\x+Friction\x*vm,*v2\p\y+Friction\y*vm)
            vect(*p\p,*p\p\x-Friction\x*2*pm,*p\p\y-Friction\y*2*pm)
         ElseIf *v1\parent\m<>0
            vect(*v1\p,*v1\p\x+Friction\x,*v1\p\y+Friction\y)
            vect(*v2\p,*v2\p\x+Friction\x,*v2\p\y+Friction\y)
         ElseIf *p\parent\m<>0 
            vect(*p\p,*p\p\x-Friction\x,*p\p\y-Friction\y)
         EndIf
         DeleteElement(collision())
      ElseIf collision()\type=2  ;type=2 : collision with halfspace
         vect(friction,collision()\colv\y,-collision()\colv\x)
         Normalize(friction)
         
         vect(v,collision()\p\oldp\x-collision()\p\p\x,collision()\p\oldp\y-collision()\p\p\y)        
         DotP=DotProduct(v,friction)
         vect(v,friction\x*DotP,friction\y*DotP)
         
         vect(friction,friction\x*collision()\p\parent\f,friction\y*collision()\p\parent\f)       
         vect(friction,friction\x/iterations,friction\y/iterations)
         
          If Abs(Length(friction))>Abs(Length(v))
            friction\x=-v\x
            friction\y=-v\y
         EndIf
            
         If  Abs(friction\x)>Abs(friction\y)
            If (v\x>0 And friction\x>0) Or (v\x<0 And friction\x<0)
               vect(friction,friction\x*-1,friction\y*-1)
            EndIf
         Else
            If (v\y>0 And friction\y>0) Or (v\y<0 And friction\y<0)
               vect(friction,friction\x*-1,friction\y*-1)
            EndIf
         EndIf
         
         vect(collision()\p\p,collision()\p\p\x-friction\x,collision()\p\p\y-friction\y)
      EndIf
   Next
   ClearList(collision())
EndProcedure


Procedure ResetCollision()
   Protected i.l
   ForEach particle()
      particle()\collision=0
   Next
   ForEach constraint()
      constraint()\collision=0
   Next
   ForEach entity()
      entity()\collision=0
   Next
EndProcedure

Procedure ProjectToAxis(*Axis.vect,*Entity.entity)
   Protected DotP.f
   Protected entity.vect
   Protected i.l
   vect(entity,*Entity\vertex[1]\p\x,*Entity\vertex[1]\p\y)
   DotP=DotProduct(*Axis,entity)
   ProjectedMin=DotP
   ProjectedMax=DotP
   For i=1 To *Entity\vertexcount
      DotP=DotProduct(*Axis,*Entity\vertex[i])
      If DotP<ProjectedMin
         ProjectedMin=DotP
      ElseIf DotP>ProjectedMax
         ProjectedMax=DotP
      EndIf
   Next
EndProcedure
Procedure ProcessCollision(*e1.entity,*e2.entity) ;here the SAT is implemented! Always keep in mind the only konvex polygons are calculated correctly
   Protected *Edge.constraint
   Protected Axis.vect
   Protected MinDistance.f
   Protected CollisionV.vect
   Protected CollAxis.vect
   Protected C.f
   Protected smallest.f
   Protected *CollEdge.constraint
   Protected Diff.vect
         
   For m=1 To *e1\edgecount+*e2\edgecount
      If m<=*e1\edgecount
         *Edge=*e1\edge[m]
      Else
         *Edge=*e2\edge[m-*e1\edgecount]
      EndIf
      vect(Axis,-(*Edge\p2\p\y-*Edge\p1\p\y),*Edge\p2\p\x-*Edge\p1\p\x)
      
      Normalize(Axis)
      
      ProjectToAxis(Axis,*e1)
      Min.f=ProjectedMin
      Max.f=ProjectedMax
      ProjectToAxis(Axis,*e2)
      
      Distance.f=IntervalDistance(Min,Max,ProjectedMin,ProjectedMax)   ;detect Collisions
      
      
      If m=1
         MinDistance=Abs(Distance)
         vect(CollAxis,Axis\x,Axis\y)
         vect(Diff,*e1\center\x-*e2\center\x,*e1\center\y-*e2\center\y)
         *CollEdge=*Edge   ;remember collision edge
      EndIf
      If Distance>0
         ProcedureReturn 0
      ElseIf Abs(Distance)<MinDistance
         MinDistance=Abs(Distance)
         vect(CollAxis,Axis\x,Axis\y)
         vect(Diff,*e1\center\x-*e2\center\x,*e1\center\y-*e2\center\y)
         *CollEdge=*Edge   ;remember collision edge
      EndIf
   Next
   vect(CollisionV,CollAxis\x*MinDistance,CollAxis\y*MinDistance)
   
   
   If *CollEdge\parent\id<>*e2\id   ;swap e1 and e2 so that e2 is the entity containing the penetrating axis
      Protected *temp.entity
      *temp=*e2
      *e2=*e1
      *e1=*temp
   EndIf
   
   C=-*e2\center\x*CollAxis\x-*e2\center\y*CollAxis\y
   Protected sign.f
   sign=sgn(*e1\center\x*CollAxis\x+*e1\center\y*CollAxis\y+C)
   If sign=1
      CollisionV\x=-CollisionV\x
      CollisionV\y=-CollisionV\y
   Else
      CollAxis\x=-CollAxis\x
      CollAxis\y=-CollAxis\y
   EndIf
   
   Protected D.f
   Protected *LowestVertex.particle 
   Protected n.l
   For n=1 To *e1\vertexcount    ;calculate penetrating vertex
      D=*e1\vertex[n]\p\x*CollAxis\x+*e1\vertex[n]\p\y*CollAxis\y+C  
      If n=1
          smallest=D
         *LowestVertex=*e1\vertex[n]
      EndIf  
      If D<smallest
         smallest=D
         *LowestVertex=*e1\vertex[n]
      EndIf
   Next
   *LowestVertex\collision+1
   
   Protected Treshold.f
   Treshold=0.001
   Protected *v1.particle
   Protected *v2.particle
   Protected Vect1.vect
   Protected Vect2.vect
   Protected o.l
   Protected *cc.constraint
   For o=1 To *e2\edgecount  ;calculate penetrating axis
     *v1=*e2\edge[o]\p1
     *v2=*e2\edge[o]\p2
     
     Vect1\y=(*v1\p\x+CollisionV\x)*CollAxis\x+(*v1\p\y+CollisionV\y)*CollAxis\y+C
     Vect2\y=(*v2\p\x+CollisionV\x)*CollAxis\x+(*v2\p\y+CollisionV\y)*CollAxis\y+C
     
     *cc=*e2\edge[o]
      If Abs(Vect1\y-Vect2\y)<Treshold
         If Abs((Vect1\y+Vect2\y)/2-smallest)<Treshold
            Goto Exit
         EndIf
      EndIf
   Next
   Exit:
   *cc\collision+1
   
   Protected col.collision  ;remember Collision for friction
   col\v1=*v1
   col\v2=*v2
   col\p=*LowestVertex
   If CollisionIdentic(col)=0
      Protected speedvect.vect
      For i= 1 To *e1\vertexcount
         speedvect\x+(*e1\vertex[i]\p\x-*e1\vertex[i]\oldp\x)
         speedvect\y+(*e1\vertex[i]\p\y-*e1\vertex[i]\oldp\y)
      Next
      vect(speedvect,speedvect\x/*e1\vertexcount,speedvect\y/*e1\vertexcount)
      Protected speedvect2.vect
      For i= 1 To *e2\vertexcount
         speedvect2\x+(*e2\vertex[i]\p\x-*e2\vertex[i]\oldp\x)
         speedvect2\y+(*e2\vertex[i]\p\y-*e2\vertex[i]\oldp\y)
      Next
      vect(speedvect2,speedvect2\x/*e2\vertexcount,speedvect2\y/*e2\vertexcount)
      vect(speedvect,speedvect\x-speedvect2\x,speedvect\y-speedvect2\y)
      Protected colspeed.vect
      vect(colspeed,CollisionV\x,CollisionV\y)
      normalize(colspeed)
      Protected DotPro.f 
      DotPro=DotProduct(colspeed,speedvect)
      vect(speedvect2,speedvect\x*DotPro,speedvect\y*DotPro)

   
      AddCollision(*v1,*v2,*LowestVertex,CollisionV)
      collision()\relativspeed=length(speedvect)
      collision()\colspeed=length(speedvect2)
   EndIf
      
   Protected Ratio.f
   If Abs(*v2\p\x-*v1\p\x)>Abs(*v2\p\y-*v1\p\y) ;don't divide by zero if vertical
      Ratio=(*LowestVertex\p\x-CollisionV\x-*v1\p\x)/(*v2\p\x-*v1\p\x)
   Else
      Ratio=(*LowestVertex\p\y-CollisionV\y-*v1\p\y)/(*v2\p\y-*v1\p\y)
   EndIf
   
   Protected mges.f
   mges=*v1\parent\m+*LowestVertex\parent\m
   Protected vm.f
   vm=*LowestVertex\parent\m/mges
   Protected pm.f
   pm=*v1\parent\m/mges
   If *e2\m<>0
      If *e1\m<>0
         *v1\p\x=*v1\p\x+CollisionV\x*(1-Ratio)*vm ;move penetrating edge
         *v1\p\y=*v1\p\y+CollisionV\y*(1-Ratio)*vm
         *v2\p\x=*v2\p\x+CollisionV\x*Ratio*vm
         *v2\p\y=*v2\p\y+CollisionV\y*Ratio*vm
      Else
         *v1\p\x=*v1\p\x+CollisionV\x*(1-Ratio)
         *v1\p\y=*v1\p\y+CollisionV\y*(1-Ratio)
         *v2\p\x=*v2\p\x+CollisionV\x*Ratio
         *v2\p\y=*v2\p\y+CollisionV\y*Ratio
      EndIf
   EndIf
   
   If *e1\m<>0
      If *e2\m<>0
         *LowestVertex\p\x=*LowestVertex\p\x-CollisionV\x*pm ;move penetrating vertex
         *LowestVertex\p\y=*LowestVertex\p\y-CollisionV\y*pm
      Else
         *LowestVertex\p\x=*LowestVertex\p\x-CollisionV\x
         *LowestVertex\p\y=*LowestVertex\p\y-CollisionV\y
      EndIf
   EndIf  
   
   ProcedureReturn 1
EndProcedure
Procedure CalcBoundingBoxes()
   Protected min.vect
   Protected max.vect
   Protected k.l
   ForEach entity()           ;calcs bounding box & center
      If entity()<>*neutralentity
         vect(entity()\center,0,0)
         vect(max,entity()\vertex[1]\p\x,entity()\vertex[1]\p\y)
         vect(min,entity()\vertex[1]\p\x,entity()\vertex[1]\p\y)
         For k=1 To entity()\vertexcount 
            entity()\center\x+entity()\vertex[k]\p\x/entity()\vertexcount
            entity()\center\y+entity()\vertex[k]\p\y/entity()\vertexcount
           
            If entity()\vertex[k]\p\x>max\x
               max\x=entity()\vertex[k]\p\x
            EndIf
            If entity()\vertex[k]\p\x<min\x
               min\x=entity()\vertex[k]\p\x
            EndIf
            If entity()\vertex[k]\p\y>max\y
               max\y=entity()\vertex[k]\p\y
            EndIf
            If entity()\vertex[k]\p\y<min\y
               min\y=entity()\vertex[k]\p\y
            EndIf
         Next
         vect(entity()\max,max\x,max\y)
         vect(entity()\min,min\x,min\y)
      EndIf
   Next 
EndProcedure

Procedure LookForCollisionRemarks()
   Protected r.l
   Protected *he.entity
   r=intelligenttolleration
   ResetCollision()              
      ForEach entity()
         *he=@entity()
         While NextElement(entity())
            If *he<>entity()
               If (*he\m<>0 Or entity()\m<>0) And entity()\id<>1 And *he\id<>1
                  If RectsOverlap(*he\min\x-r,*he\min\y-r,*he\max\x+r,*he\max\y+r,entity()\min\x-r,entity()\min\y-r,entity()\max\x+r,entity()\max\y+r)
                      AddCollisionRemark(*he,entity())
                  EndIf
               EndIf
            EndIf
         Wend
         ChangeCurrentElement(entity(), *he)
     Next   
EndProcedure
Procedure StaisfyConstraints()
    Protected r.l
    r=intelligenttolleration
    ResetList(CollisionRemark())
    While NextElement(CollisionRemark())
       If RectsOverlap(CollisionRemark()\e1\min\x-r,CollisionRemark()\e1\min\y-r,CollisionRemark()\e1\max\x+r,CollisionRemark()\e1\max\y+r,CollisionRemark()\e2\min\x-r,CollisionRemark()\e2\min\y-r,CollisionRemark()\e2\max\x+r,CollisionRemark()\e2\max\y+r)
          CollisionRemark()\e1\collision+ProcessCollision(CollisionRemark()\e1,CollisionRemark()\e2)
       Else
          DeleteElement(CollisionRemark())
       EndIf
    Wend
EndProcedure

Procedure setstaticposition() ; makes static objects static...
   ForEach particle()
      If particle()\parent\m=0
         vect(particle()\oldp,particle()\p\x,particle()\p\y)
      EndIf
   Next
EndProcedure

Procedure CalcStep() ;the main calculation Procedure, the whole physic world is calculated in here
   ;order is Important!
   CountAll()   
   UpdateVerlet()
   For i=1 To iterations
      UpdateSpeed()
      CalcBoundingBoxes()
      UpdateConstraints()
      UpdateParticleHalfspaceCollision()  
      If intelligentiterations=0 
         If i=1 ;use first iterations To detected which entities are near each other
             LookForCollisionRemarks() 
         EndIf
         StaisfyConstraints()   ;just test the entities, which were near to each other in the first iteration
      EndIf
      UpdateFriction()   
   Next  
   setstaticposition()
   ClearList(CollisionRemark())     
EndProcedure
; IDE Options = PureBasic 4.61 (Windows - x86)
; CursorPosition = 605
; FirstLine = 97
; Folding = AAAAA9
; EnableXP